# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/geneorder',
                'doc_host': 'https://galicae.github.io',
                'git_url': 'https://github.com/galicae/geneorder',
                'lib_path': 'geneorder'},
  'syms': { 'geneorder.core': { 'geneorder.core.plot_synteny': ('core.html#plot_synteny', 'geneorder/core.py'),
                                'geneorder.core.plot_synteny_schematic': ('core.html#plot_synteny_schematic', 'geneorder/core.py')},
            'geneorder.util': { 'geneorder.util._flip_strand': ('util.html#_flip_strand', 'geneorder/util.py'),
                                'geneorder.util.decorate': ('util.html#decorate', 'geneorder/util.py'),
                                'geneorder.util.estimate_plot_size': ('util.html#estimate_plot_size', 'geneorder/util.py'),
                                'geneorder.util.filter': ('util.html#filter', 'geneorder/util.py'),
                                'geneorder.util.flip': ('util.html#flip', 'geneorder/util.py'),
                                'geneorder.util.gff_attribute_selector': ('util.html#gff_attribute_selector', 'geneorder/util.py'),
                                'geneorder.util.insert_break': ('util.html#insert_break', 'geneorder/util.py'),
                                'geneorder.util.insert_gap': ('util.html#insert_gap', 'geneorder/util.py'),
                                'geneorder.util.read_aln': ('util.html#read_aln', 'geneorder/util.py'),
                                'geneorder.util.read_gff': ('util.html#read_gff', 'geneorder/util.py'),
                                'geneorder.util.syntenic_block_borders': ('util.html#syntenic_block_borders', 'geneorder/util.py')}}}
